@@ -50,6 +50,12 @@
  * function names, etc. To add a provider, call
  * [method@Completion.add_provider].
  *
+ * When several providers match, they are all shown in the completion
+ * window, but one can switch between providers: see the
+ * #GtkSourceCompletion::move-page signal. It is also possible to
+ * activate the first proposals with key bindings, see the
+ * #GtkSourceCompletion:accelerators property.
+ *
  * The [iface@CompletionProposal] interface represents a proposal.
  *
  * If a proposal contains extra information (see
@@ -143,11 +149,14 @@ struct _GtkSourceCompletion
 	guint page_size;
 
 	/* Handler for gtk_widget_add_tick_callback() to do delayed calls to
-	 * gtk_widget_set_visible(FALSE) (so that we don't potentially flap between
+	 * gtk_widget_hide() (so that we don't potentially flap between
 	 * hide/show while typing.
 	 */
 	guint hide_tick_handler;
 
+	/* If we're currently being displayed */
+	guint shown : 1;
+
 	/* If we have a completion actively in play */
 	guint waiting_for_results : 1;
 
@@ -204,10 +213,7 @@ display_show (GtkSourceCompletion *self)
 		self->hide_tick_handler = 0;
 	}
 
-	if (gtk_widget_get_mapped (GTK_WIDGET (self->view)))
-	{
-		gtk_widget_set_visible (GTK_WIDGET (_gtk_source_completion_get_display (self)), TRUE);
-	}
+	gtk_widget_show (GTK_WIDGET (_gtk_source_completion_get_display (self)));
 }
 
 static gboolean
@@ -225,7 +231,7 @@ display_hide_cb (GtkWidget     *widget,
 
 	if (self->display != NULL)
 	{
-		gtk_widget_set_visible (GTK_WIDGET (self->display), FALSE);
+		gtk_widget_hide (GTK_WIDGET (self->display));
 	}
 
 	return G_SOURCE_REMOVE;
@@ -419,7 +425,7 @@ gtk_source_completion_cancel (GtkSourceCompletion *self)
 		if (self->display != NULL)
 		{
 			_gtk_source_completion_list_set_context (self->display, NULL);
-			gtk_widget_set_visible (GTK_WIDGET (self->display), FALSE);
+			gtk_widget_hide (GTK_WIDGET (self->display));
 		}
 	}
 }
@@ -467,8 +473,7 @@ gtk_source_completion_compute_bounds (GtkSourceCompletion *self,
 
 static void
 gtk_source_completion_start (GtkSourceCompletion           *self,
-                             GtkSourceCompletionActivation  activation,
-                             gboolean                       from_trigger)
+                             GtkSourceCompletionActivation  activation)
 {
 	GtkSourceCompletionContext *context = NULL;
 	GtkTextBuffer *buffer;
@@ -482,11 +487,8 @@ gtk_source_completion_start (GtkSourceCompletion           *self,
 
 	if (!gtk_source_completion_compute_bounds (self, &begin, &end))
 	{
-		if (!from_trigger && activation == GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE)
-		{
+		if (activation == GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE)
 			goto cleanup;
-		}
-
 		begin = end;
 	}
 
@@ -529,8 +531,7 @@ cleanup:
 
 static void
 gtk_source_completion_update (GtkSourceCompletion           *self,
-                              GtkSourceCompletionActivation  activation,
-                              gboolean                       from_trigger)
+                              GtkSourceCompletionActivation  activation)
 {
 	GtkTextBuffer *buffer;
 	GtkTextMark *insert;
@@ -615,7 +616,7 @@ gtk_source_completion_update (GtkSourceCompletion           *self,
 
 reset:
 	gtk_source_completion_cancel (self);
-	gtk_source_completion_start (self, activation, from_trigger);
+	gtk_source_completion_start (self, activation);
 }
 
 static void
@@ -625,7 +626,7 @@ gtk_source_completion_real_hide (GtkSourceCompletion *self)
 
 	if (self->display != NULL)
 	{
-		gtk_widget_set_visible (GTK_WIDGET (self->display), FALSE);
+		gtk_widget_hide (GTK_WIDGET (self->display));
 	}
 }
 
@@ -638,14 +639,14 @@ gtk_source_completion_real_show (GtkSourceCompletion *self)
 
 	display = _gtk_source_completion_get_display (self);
 
-	/* If the user is requesting completion manually, we should throw away
-	 * our previous results and attempt completion over. Otherwise, providers
-	 * which bailed because they were in _INTERACTIVE_ mode will not be
-	 * requeried for updated results.
-	 */
-	g_clear_object (&self->context);
-
-	gtk_source_completion_start (self, GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED, FALSE);
+	if (self->context == NULL)
+	{
+		gtk_source_completion_start (self, GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED);
+	}
+	else
+	{
+		gtk_source_completion_update (self, GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED);
+	}
 
 	_gtk_source_completion_list_set_context (display, self->context);
 
@@ -669,9 +670,7 @@ gtk_source_completion_queued_update_cb (gpointer user_data)
 	self->queued_update = 0;
 
 	if (self->context != NULL)
-	{
-		gtk_source_completion_update (self, GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE, FALSE);
-	}
+		gtk_source_completion_update (self, GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE);
 
 	return G_SOURCE_REMOVE;
 }
@@ -804,8 +803,7 @@ gtk_source_completion_buffer_insert_text_after_cb (GtkSourceCompletion *self,
                                                    gint                 len,
                                                    GtkTextBuffer       *buffer)
 {
-	const GtkSourceCompletionActivation activation = GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE;
-	gboolean from_trigger = FALSE;
+	GtkSourceCompletionActivation activation = GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE;
 	GtkTextIter begin;
 	GtkTextIter end;
 
@@ -842,8 +840,8 @@ gtk_source_completion_buffer_insert_text_after_cb (GtkSourceCompletion *self,
 					 * completion. We need to cancel the previous completion (if any) first
 					 * and then try to start a new completion due to trigger.
 					 */
-					from_trigger = TRUE;
 					gtk_source_completion_cancel (self);
+					activation = GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE;
 					goto do_completion;
 				}
 			}
@@ -856,9 +854,9 @@ gtk_source_completion_buffer_insert_text_after_cb (GtkSourceCompletion *self,
 do_completion:
 
 	if (self->context == NULL)
-		gtk_source_completion_start (self, activation, from_trigger);
+		gtk_source_completion_start (self, activation);
 	else
-		gtk_source_completion_update (self, activation, from_trigger);
+		gtk_source_completion_update (self, activation);
 }
 
 static void
@@ -1685,24 +1683,3 @@ _gtk_source_completion_css_changed (GtkSourceCompletion *self,
 		_gtk_source_completion_list_set_font_desc (self->display, self->font_desc);
 	}
 }
-
-gboolean
-_gtk_source_completion_get_visible (GtkSourceCompletion *self)
-{
-	g_return_val_if_fail (GTK_SOURCE_IS_COMPLETION (self), FALSE);
-
-	return self->display && gtk_widget_get_visible (GTK_WIDGET (self->display));
-}
-
-void
-_gtk_source_completion_move_cursor (GtkSourceCompletion *self,
-                                    GtkMovementStep      step,
-                                    int                  direction)
-{
-	g_return_if_fail (GTK_SOURCE_IS_COMPLETION (self));
-
-	if (self->display != NULL)
-	{
-		_gtk_source_completion_list_move_cursor (self->display, step, direction);
-	}
-}
