@@ -23,8 +23,6 @@
 
 #include <string.h>
 
-#include "gtksourceannotation-private.h"
-#include "gtksourceannotationprovider.h"
 #include "gtksourceassistant-private.h"
 #include "gtksourcehoverassistant-private.h"
 #include "gtksourcehovercontext-private.h"
@@ -77,7 +75,6 @@ gtk_source_hover_assistant_should_dismiss (GtkSourceHoverAssistant *self)
 		GdkRectangle popup_area, root_area;
 		double popup_x, popup_y;
 		double transform_x, transform_y;
-		GtkWidget *view;
 		GtkRoot *root;
 
 		popup_x = gdk_popup_get_position_x (GDK_POPUP (surface));
@@ -100,34 +97,6 @@ gtk_source_hover_assistant_should_dismiss (GtkSourceHoverAssistant *self)
 		{
 			return FALSE;
 		}
-
-		/* If our cursor is still within the hovered_at area we
-		 * do not want to dismiss yet either.
-		 */
-		if ((view = gtk_widget_get_ancestor (GTK_WIDGET (self), GTK_SOURCE_TYPE_VIEW)))
-		{
-			graphene_point_t point;
-			int buffer_x;
-			int buffer_y;
-
-			if (gtk_widget_compute_point (GTK_WIDGET (root),
-			                              view,
-			                              &GRAPHENE_POINT_INIT (self->root_x, self->root_y),
-			                              &point))
-			{
-				gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (view),
-				                                       GTK_TEXT_WINDOW_WIDGET,
-				                                       point.x,
-				                                       point.y,
-				                                       &buffer_x,
-				                                       &buffer_y);
-
-				if (gdk_rectangle_contains_point (&self->hovered_at, buffer_x, buffer_y))
-				{
-					return FALSE;
-				}
-			}
-		}
 	}
 
 	return TRUE;
@@ -314,8 +283,6 @@ static void
 gtk_source_hover_assistant_get_target_location (GtkSourceAssistant *assistant,
                                                 GdkRectangle       *rect)
 {
-	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-
 	GtkSourceHoverAssistant *self = GTK_SOURCE_HOVER_ASSISTANT (assistant);
 	GtkStyleContext *style_context;
 	GtkBorder padding;
@@ -326,8 +293,6 @@ gtk_source_hover_assistant_get_target_location (GtkSourceAssistant *assistant,
 	gtk_style_context_get_padding (style_context, &padding);
 
 	rect->x -= padding.left;
-
-	G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 static void
@@ -349,32 +314,6 @@ gtk_source_hover_assistant_dispose (GObject *object)
 	G_OBJECT_CLASS (gtk_source_hover_assistant_parent_class)->dispose (object);
 }
 
-static void
-gtk_source_hover_assistant_click_pressed_cb (GtkSourceHoverAssistant *self,
-                                             int                      n_press,
-                                             double                   x,
-                                             double                   y,
-                                             GtkGestureClick         *click)
-{
-	GdkEventSequence *sequence;
-	GdkEvent *event;
-
-	g_assert (GTK_SOURCE_IS_HOVER_ASSISTANT (self));
-	g_assert (GTK_IS_GESTURE_CLICK (click));
-
-	sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (click));
-	event = gtk_gesture_get_last_event (GTK_GESTURE (click), sequence);
-
-	/* WORKAROUND: See comment below in gtk_source_hover_assistant_init().
-	 * We have to block context menus from here until we can be sure they'll
-	 * work with GtkPopover:autohide disabled.
-	 */
-	if (gdk_event_triggers_context_menu (event))
-	{
-		gtk_gesture_set_state (GTK_GESTURE (click), GTK_EVENT_SEQUENCE_CLAIMED);
-	}
-}
-
 static void
 gtk_source_hover_assistant_class_init (GtkSourceHoverAssistantClass *klass)
 {
@@ -395,13 +334,10 @@ gtk_source_hover_assistant_class_init (GtkSourceHoverAssistantClass *klass)
 static void
 gtk_source_hover_assistant_init (GtkSourceHoverAssistant *self)
 {
-	GtkEventController *click;
 	GtkEventController *scroll;
 
 	gtk_widget_add_css_class (GTK_WIDGET (self), "hover-assistant");
 
-	_gtk_source_assistant_set_pref_position (GTK_SOURCE_ASSISTANT (self), GTK_POS_TOP);
-
 	gtk_popover_set_autohide (GTK_POPOVER (self), FALSE);
 	gtk_popover_set_position (GTK_POPOVER (self), GTK_POS_TOP);
 
@@ -424,21 +360,6 @@ gtk_source_hover_assistant_init (GtkSourceHoverAssistant *self)
 	                         self,
 	                         G_CONNECT_SWAPPED);
 	gtk_widget_add_controller (GTK_WIDGET (self), g_object_ref (self->popover_motion));
-
-	/* WORKAROUND: Until we have a way to ensure that showing context
-	 * menus from the popover won't break our popover, we need to prevent
-	 * them from potentially breaking input/grabs.
-	 */
-	click = GTK_EVENT_CONTROLLER (gtk_gesture_click_new ());
-	g_signal_connect_object (click,
-	                         "pressed",
-	                         G_CALLBACK (gtk_source_hover_assistant_click_pressed_cb),
-	                         self,
-	                         G_CONNECT_SWAPPED);
-	gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (click), 0);
-	gtk_gesture_single_set_exclusive (GTK_GESTURE_SINGLE (click), TRUE);
-	gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (click), GTK_PHASE_CAPTURE);
-	gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (click));
 }
 
 GtkSourceAssistant *
@@ -464,11 +385,8 @@ gtk_source_hover_assistant_populate_cb (GObject      *object,
 	{
 		if (!self->disposed)
 		{
-			GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (self));
-			gboolean mapped = parent && gtk_widget_get_mapped (parent);
-			gboolean empty = _gtk_source_hover_display_is_empty (self->display);
-
-			gtk_widget_set_visible (GTK_WIDGET (self), mapped && !empty);
+			gtk_widget_set_visible (GTK_WIDGET (self),
+			                        !_gtk_source_hover_display_is_empty (self->display));
 		}
 	}
 
@@ -504,7 +422,7 @@ _gtk_source_hover_assistant_display (GtkSourceHoverAssistant  *self,
 
 	if (n_providers == 0)
 	{
-		gtk_widget_set_visible (GTK_WIDGET (self), FALSE);
+		gtk_widget_hide (GTK_WIDGET (self));
 		return;
 	}
 
@@ -518,7 +436,7 @@ _gtk_source_hover_assistant_display (GtkSourceHoverAssistant  *self,
 
 	if (!gdk_rectangle_intersect (&location_rect, &visible_rect, &location_rect))
 	{
-		gtk_widget_set_visible (GTK_WIDGET (self), FALSE);
+		gtk_widget_hide (GTK_WIDGET (self));
 		return;
 	}
 
@@ -544,83 +462,6 @@ _gtk_source_hover_assistant_display (GtkSourceHoverAssistant  *self,
 	g_object_unref (context);
 }
 
-static void
-gtk_source_hover_assistant_populate_annotation_cb (GObject      *object,
-                                                   GAsyncResult *result,
-                                                   gpointer      user_data)
-{
-	GtkSourceAnnotationProvider *provider = (GtkSourceAnnotationProvider *)object;
-	GtkSourceHoverAssistant *self = user_data;
-	GError *error = NULL;
-
-	g_assert (GTK_SOURCE_IS_ANNOTATION_PROVIDER (provider));
-	g_assert (G_IS_ASYNC_RESULT (result));
-	g_assert (GTK_SOURCE_IS_HOVER_ASSISTANT (self));
-
-	if (gtk_source_annotation_provider_populate_hover_finish (provider, result, &error))
-	{
-		if (!self->disposed)
-		{
-			GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (self));
-			gboolean mapped = parent && gtk_widget_get_mapped (parent);
-			gboolean empty = _gtk_source_hover_display_is_empty (self->display);
-
-			gtk_widget_set_visible (GTK_WIDGET (self), mapped && !empty);
-		}
-	}
-
-	g_clear_object (&self);
-	g_clear_error (&error);
-}
-
-void
-_gtk_source_hover_assistant_display_annotation (GtkSourceHoverAssistant     *self,
-                                                GtkSourceAnnotationProvider *provider,
-                                                GtkSourceAnnotation         *annotation)
-{
-	GtkSourceView *view;
-	GdkRectangle location_rect;
-	GdkRectangle visible_rect;
-	int buffer_x, buffer_y;
-
-	memset (&self->hovered_at, 0, sizeof self->hovered_at);
-
-	_gtk_source_annotation_get_rect (annotation, &location_rect);
-
-	view = GTK_SOURCE_VIEW (gtk_widget_get_parent (GTK_WIDGET (self)));
-
-	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (view),
-	                                       GTK_TEXT_WINDOW_TEXT,
-	                                       location_rect.x,
-	                                       location_rect.y,
-	                                       &buffer_x,
-	                                       &buffer_y);
-	location_rect.x = buffer_x;
-	location_rect.y = buffer_y;
-
-	gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (view), &visible_rect);
-
-	if (!gdk_rectangle_intersect (&location_rect, &visible_rect, &location_rect))
-	{
-		gtk_widget_set_visible (GTK_WIDGET (self), FALSE);
-		return;
-	}
-
-	self->hovered_at = location_rect;
-
-	_gtk_source_hover_display_clear (self->display);
-
-	g_clear_object (&self->cancellable);
-	self->cancellable = g_cancellable_new ();
-
-	gtk_source_annotation_provider_populate_hover_async (provider,
-	                                                     annotation,
-	                                                     self->display,
-	                                                     self->cancellable,
-	                                                     gtk_source_hover_assistant_populate_annotation_cb,
-	                                                     g_object_ref (self));
-}
-
 void
 _gtk_source_hover_assistant_dismiss (GtkSourceHoverAssistant *self)
 {
@@ -629,7 +470,7 @@ _gtk_source_hover_assistant_dismiss (GtkSourceHoverAssistant *self)
 	g_cancellable_cancel (self->cancellable);
 	g_clear_object (&self->cancellable);
 
-	gtk_widget_set_visible (GTK_WIDGET (self), FALSE);
+	gtk_widget_hide (GTK_WIDGET (self));
 
 	_gtk_source_hover_display_clear (self->display);
 }
