@@ -73,15 +73,10 @@ gtk_source_view_snippets_scroll_to_insert (GtkSourceViewSnippets *snippets)
 	GtkTextIter iter;
 	GdkRectangle area;
 	GdkRectangle visible_rect;
-	guint top_margin;
 	double x, y;
 
 	g_assert (snippets != NULL);
 
-	g_object_get (snippets->view,
-		      "top-margin", &top_margin,
-		      NULL);
-
 	mark = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (snippets->buffer));
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (snippets->buffer), &iter, mark);
 	gtk_text_view_get_iter_location (GTK_TEXT_VIEW (snippets->view), &iter, &area);
@@ -102,7 +97,7 @@ gtk_source_view_snippets_scroll_to_insert (GtkSourceViewSnippets *snippets)
 		y = visible_rect.y;
 
 	gtk_adjustment_set_value (gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (snippets->view)), x);
-	gtk_adjustment_set_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (snippets->view)), y + top_margin);
+	gtk_adjustment_set_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (snippets->view)), y);
 
 	gtk_source_view_snippets_update_informative (snippets);
 }
@@ -251,7 +246,7 @@ buffer_cursor_moved_cb (GtkSourceBuffer       *buffer,
 		{
 			if (snippets->informative != NULL)
 			{
-				gtk_widget_set_visible (GTK_WIDGET (snippets->informative), FALSE);
+				gtk_widget_hide (GTK_WIDGET (snippets->informative));
 			}
 		}
 	}
@@ -368,14 +363,11 @@ gtk_source_view_snippets_update_informative (GtkSourceViewSnippets *snippets)
 
 	if (gtk_widget_get_visible (GTK_WIDGET (snippets->informative)))
 	{
-		_gtk_source_assistant_update_position (GTK_SOURCE_ASSISTANT (snippets->informative));
+		_gtk_source_assistant_set_needs_position (GTK_SOURCE_ASSISTANT (snippets->informative));
 	}
 	else
 	{
-		if (gtk_widget_get_mapped (GTK_WIDGET (snippets->view)))
-		{
-			gtk_widget_set_visible (GTK_WIDGET (snippets->informative), TRUE);
-		}
+		gtk_widget_show (GTK_WIDGET (snippets->informative));
 	}
 
 	return;
@@ -383,7 +375,7 @@ gtk_source_view_snippets_update_informative (GtkSourceViewSnippets *snippets)
 hide_informative:
 	if (snippets->informative != NULL)
 	{
-		gtk_widget_set_visible (GTK_WIDGET (snippets->informative), FALSE);
+		gtk_widget_hide (GTK_WIDGET (snippets->informative));
 	}
 }
 
@@ -573,7 +565,7 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 	 * the view, such as with completion providers.
 	 */
 
-	if (snippets->buffer == NULL)
+	if (snippets->buffer == NULL || snippets->queue.length == 0)
 	{
 		return GDK_EVENT_PROPAGATE;
 	}
@@ -598,8 +590,6 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 		 */
 		if (snippet != NULL)
 		{
-			_gtk_source_view_hide_completion (snippets->view);
-
 			if ((state & modifiers) == 0)
 			{
 				if (!_gtk_source_snippet_move_next (snippet))
@@ -636,8 +626,6 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 		    !has_selection &&
 		    gtk_source_view_snippets_try_expand (snippets, &end))
 		{
-			_gtk_source_view_hide_completion (snippets->view);
-
 			gtk_source_view_snippets_scroll_to_insert (snippets);
 
 			ret = GDK_EVENT_STOP;
@@ -646,11 +634,11 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 	}
 
 cleanup:
-	if (ret != GDK_EVENT_PROPAGATE && snippets->queue.length == 0)
+	if (snippets->queue.length == 0)
 	{
 		if (snippets->informative)
 		{
-			gtk_widget_set_visible (GTK_WIDGET (snippets->informative), FALSE);
+			gtk_widget_hide (GTK_WIDGET (snippets->informative));
 		}
 	}
 
@@ -739,7 +727,7 @@ _gtk_source_view_snippets_pop (GtkSourceViewSnippets *snippets)
 	{
 		if (snippets->informative != NULL)
 		{
-			gtk_widget_set_visible (GTK_WIDGET (snippets->informative), FALSE);
+			gtk_widget_hide (GTK_WIDGET (snippets->informative));
 		}
 	}
 }
